@@ -13,12 +13,12 @@
     license reproduced at the end of this file.
 =============================================================================*/
 
-// Tests error reporting: #ifdef, #ifndef syntax errors.
+// Tests error reporting: Redefinitions of macros.
+#define FTN_LIKE(a)     ( a )
 
-// 15.5: No argument. 
-//E t_6_022.cpp(20): error: ill formed preprocessor directive: #ifndef
-#ifndef
-#endif
+// 19.5: different parameter usage
+//E t_6_041.cpp(21): warning: illegal macro redefinition: FTN_LIKE
+#define FTN_LIKE(b)     ( a )
 
 /*-
  * Copyright (c) 1998, 2002-2005 Kiyoshi Matsui <kmatsui@t3.rim.or.jp>
