@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-* Copyright (C) 1997-2008, International Business Machines Corporation and    *
+* Copyright (C) 1997-2010, International Business Machines Corporation and    *
 * others. All Rights Reserved.                                                *
 *******************************************************************************
 *
@@ -33,6 +33,8 @@
 #include "unicode/ustring.h"
 #include "unicode/ucurr.h"
 #include "unicode/curramt.h"
+#include "unicode/numsys.h"
+#include "unicode/rbnf.h"
 #include "winnmfmt.h"
 #include "uresimp.h"
 #include "uhash.h"
@@ -41,6 +43,8 @@
 #include "ucln_in.h"
 #include "cstring.h"
 #include "putilimp.h"
+#include "umutex.h"
+#include "digitlst.h"
 #include <float.h>
 
 //#define FMT_DEBUG
@@ -72,6 +76,17 @@ static const UChar gLastResortPercentPat[] = {
 static const UChar gLastResortScientificPat[] = {
     0x23, 0x45, 0x30, 0 /* "#E0" */
 };
+static const UChar gLastResortIsoCurrencyPat[] = {
+    0xA4, 0xA4, 0x23, 0x30, 0x2E, 0x30, 0x30, 0x3B, 0x28, 0xA4, 0xA4, 0x23, 0x30, 0x2E, 0x30, 0x30, 0x29, 0 /* "\u00A4\u00A4#0.00;(\u00A4\u00A4#0.00)" */
+};
+static const UChar gLastResortPluralCurrencyPat[] = {
+    0x23, 0x30, 0x2E, 0x30, 0x30, 0xA0, 0xA4, 0xA4, 0xA4, 0 /* "#0.00\u00A0\u00A4\u00A4\u00A4*/
+};
+
+static const UChar gSingleCurrencySign[] = {0xA4, 0};
+static const UChar gDoubleCurrencySign[] = {0xA4, 0xA4, 0};
+
+static const UChar gSlash = 0x2f;
 
 // If the maximum base 10 exponent were 4, then the largest number would
 // be 99,999 which has 5 digits.
@@ -84,9 +99,46 @@ static const UChar * const gLastResortNumberPatterns[] =
     gLastResortDecimalPat,
     gLastResortCurrencyPat,
     gLastResortPercentPat,
-    gLastResortScientificPat
+    gLastResortScientificPat,
+    gLastResortIsoCurrencyPat,
+    gLastResortPluralCurrencyPat,
 };
 
+// Static hashtable cache of NumberingSystem objects used by NumberFormat
+static UHashtable * NumberingSystem_cache = NULL;
+
+static UMTX nscacheMutex = NULL;
+
+#if !UCONFIG_NO_SERVICE
+static U_NAMESPACE_QUALIFIER ICULocaleService* gService = NULL;
+#endif
+
+/**
+ * Release all static memory held by Number Format.
+ */
+U_CDECL_BEGIN
+static void U_CALLCONV
+deleteNumberingSystem(void *obj) {
+    delete (U_NAMESPACE_QUALIFIER NumberingSystem *)obj;
+}
+
+static UBool U_CALLCONV numfmt_cleanup(void) {
+#if !UCONFIG_NO_SERVICE
+    if (gService) {
+        delete gService;
+        gService = NULL;
+    }
+#endif
+    if (NumberingSystem_cache) {
+        // delete NumberingSystem_cache;
+        uhash_close(NumberingSystem_cache);
+        NumberingSystem_cache = NULL;
+    }
+
+    return TRUE;
+}
+U_CDECL_END
+
 // *****************************************************************************
 // class NumberFormat
 // *****************************************************************************
@@ -112,7 +164,7 @@ UBool SimpleNumberFormatFactory::visible(void) const {
 }
 
 const UnicodeString *
-SimpleNumberFormatFactory::getSupportedIDs(int32_t &count, UErrorCode& status) const 
+SimpleNumberFormatFactory::getSupportedIDs(int32_t &count, UErrorCode& status) const
 {
     if (U_SUCCESS(status)) {
         count = 1;
@@ -159,6 +211,7 @@ NumberFormat::operator=(const NumberFormat& rhs)
 {
     if (this != &rhs)
     {
+        Format::operator=(rhs);
         fGroupingUsed = rhs.fGroupingUsed;
         fMaxIntegerDigits = rhs.fMaxIntegerDigits;
         fMinIntegerDigits = rhs.fMinIntegerDigits;
@@ -208,7 +261,7 @@ NumberFormat::operator==(const Format& that) const
         if (first) { printf("[ "); first = FALSE; } else { printf(", "); }
         debug("fCurrency !=");
     }
-    if (!first) { printf(" ]"); }    
+    if (!first) { printf(" ]"); }
 #endif
 
     return ((this == &that) ||
@@ -222,10 +275,165 @@ NumberFormat::operator==(const Format& that) const
               u_strcmp(fCurrency, other->fCurrency) == 0)));
 }
 
-// -------------------------------------x
+// -------------------------------------
+// Default implementation sets unsupported error; subclasses should
+// override.
+
+UnicodeString&
+NumberFormat::format(double /* unused number */,
+                     UnicodeString& toAppendTo,
+                     FieldPositionIterator* /* unused posIter */,
+                     UErrorCode& status) const
+{
+    if (!U_FAILURE(status)) {
+        status = U_UNSUPPORTED_ERROR;
+    }
+    return toAppendTo;
+}
+
+// -------------------------------------
+// Default implementation sets unsupported error; subclasses should
+// override.
+
+UnicodeString&
+NumberFormat::format(int32_t /* unused number */,
+                     UnicodeString& toAppendTo,
+                     FieldPositionIterator* /* unused posIter */,
+                     UErrorCode& status) const
+{
+    if (!U_FAILURE(status)) {
+        status = U_UNSUPPORTED_ERROR;
+    }
+    return toAppendTo;
+}
+
+// -------------------------------------
+// Default implementation sets unsupported error; subclasses should
+// override.
+
+UnicodeString&
+NumberFormat::format(int64_t /* unused number */,
+                     UnicodeString& toAppendTo,
+                     FieldPositionIterator* /* unused posIter */,
+                     UErrorCode& status) const
+{
+    if (!U_FAILURE(status)) {
+        status = U_UNSUPPORTED_ERROR;
+    }
+    return toAppendTo;
+}
+
+// -------------------------------------
+// Decimal Number format() default implementation 
+// Subclasses do not normally override this function, but rather the DigitList
+// formatting functions..
+//   The expected call chain from here is
+//      this function ->
+//      NumberFormat::format(Formattable  ->
+//      DecimalFormat::format(DigitList    
+//
+//   Or, for subclasses of Formattable that do not know about DigitList,
+//       this Function ->
+//       NumberFormat::format(Formattable  ->
+//       NumberFormat::format(DigitList  ->
+//       XXXFormat::format(double
+
+UnicodeString&
+NumberFormat::format(const StringPiece &decimalNum,
+                     UnicodeString& toAppendTo,
+                     FieldPositionIterator* fpi,
+                     UErrorCode& status) const
+{
+    Formattable f;
+    f.setDecimalNumber(decimalNum, status);
+    format(f, toAppendTo, fpi, status);
+    return toAppendTo;
+}
+
+// -------------------------------------
 // Formats the number object and save the format
 // result in the toAppendTo string buffer.
 
+// utility to save/restore state, used in two overloads
+// of format(const Formattable&...) below.
+
+class ArgExtractor {
+  NumberFormat *ncnf;
+  const Formattable* num;
+  UBool setCurr;
+  UChar save[4];
+
+ public:
+  ArgExtractor(const NumberFormat& nf, const Formattable& obj, UErrorCode& status);
+  ~ArgExtractor();
+
+  const Formattable* number(void) const;
+};
+
+inline const Formattable*
+ArgExtractor::number(void) const {
+  return num;
+}
+
+ArgExtractor::ArgExtractor(const NumberFormat& nf, const Formattable& obj, UErrorCode& status)
+    : ncnf((NumberFormat*) &nf), num(&obj), setCurr(FALSE) {
+
+    const UObject* o = obj.getObject(); // most commonly o==NULL
+    if (o != NULL &&
+        o->getDynamicClassID() == CurrencyAmount::getStaticClassID()) {
+        // getISOCurrency() returns a pointer to internal storage, so we
+        // copy it to retain it across the call to setCurrency().
+        const CurrencyAmount* amt = (const CurrencyAmount*) o;
+        const UChar* curr = amt->getISOCurrency();
+        u_strcpy(save, nf.getCurrency());
+        setCurr = (u_strcmp(curr, save) != 0);
+        if (setCurr) {
+            ncnf->setCurrency(curr, status);
+        }
+        num = &amt->getNumber();
+    }
+}
+
+ArgExtractor::~ArgExtractor() {
+    if (setCurr) {
+        UErrorCode ok = U_ZERO_ERROR;
+        ncnf->setCurrency(save, ok); // always restore currency
+    }
+}
+
+UnicodeString& NumberFormat::format(const DigitList &number,
+                      UnicodeString& appendTo,
+                      FieldPositionIterator* posIter,
+                      UErrorCode& status) const {
+    // DecimalFormat overrides this function, and handles DigitList based big decimals.
+    // Other subclasses (ChoiceFormat, RuleBasedNumberFormat) do not (yet) handle DigitLists,
+    // so this default implementation falls back to formatting decimal numbers as doubles.
+    if (U_FAILURE(status)) {
+        return appendTo;
+    }
+    double dnum = number.getDouble();
+    format(dnum, appendTo, posIter, status);
+    return appendTo;
+}
+
+
+
+UnicodeString&
+NumberFormat::format(const DigitList &number,
+                     UnicodeString& appendTo,
+                     FieldPosition& pos,
+                     UErrorCode &status) const { 
+    // DecimalFormat overrides this function, and handles DigitList based big decimals.
+    // Other subclasses (ChoiceFormat, RuleBasedNumberFormat) do not (yet) handle DigitLists,
+    // so this default implementation falls back to formatting decimal numbers as doubles.
+    if (U_FAILURE(status)) {
+        return appendTo;
+    }
+    double dnum = number.getDouble();
+    format(dnum, appendTo, pos, status);
+    return appendTo;
+}
+
 UnicodeString&
 NumberFormat::format(const Formattable& obj,
                         UnicodeString& appendTo,
@@ -234,51 +442,80 @@ NumberFormat::format(const Formattable& obj,
 {
     if (U_FAILURE(status)) return appendTo;
 
-    NumberFormat* nonconst = (NumberFormat*) this;
-    const Formattable* n = &obj;
+    ArgExtractor arg(*this, obj, status);
+    const Formattable *n = arg.number();
 
-    UChar save[4];
-    UBool setCurr = FALSE;
-    const UObject* o = obj.getObject(); // most commonly o==NULL
-    if (o != NULL &&
-        o->getDynamicClassID() == CurrencyAmount::getStaticClassID()) {
-        // getISOCurrency() returns a pointer to internal storage, so we
-        // copy it to retain it across the call to setCurrency().
-        const CurrencyAmount* amt = (const CurrencyAmount*) o;
-        const UChar* curr = amt->getISOCurrency();
-        u_strcpy(save, getCurrency());
-        setCurr = (u_strcmp(curr, save) != 0);
-        if (setCurr) {
-            nonconst->setCurrency(curr, status);
+    if (n->isNumeric() && n->getDigitList() != NULL) {
+        // Decimal Number.  We will have a DigitList available if the value was
+        //   set to a decimal number, or if the value originated with a parse.
+        //
+        // The default implementation for formatting a DigitList converts it
+        // to a double, and formats that, allowing formatting classes that don't
+        // know about DigitList to continue to operate as they had.
+        //
+        // DecimalFormat overrides the DigitList formatting functions.
+        format(*n->getDigitList(), appendTo, pos, status);
+    } else {
+        switch (n->getType()) {
+        case Formattable::kDouble:
+            format(n->getDouble(), appendTo, pos);
+            break;
+        case Formattable::kLong:
+            format(n->getLong(), appendTo, pos);
+            break;
+        case Formattable::kInt64:
+            format(n->getInt64(), appendTo, pos);
+            break;
+        default:
+            status = U_INVALID_FORMAT_ERROR;
+            break;
         }
-        n = &amt->getNumber();
     }
 
-    switch (n->getType()) {
-    case Formattable::kDouble:
-        format(n->getDouble(), appendTo, pos);
-        break;
-    case Formattable::kLong:
-        format(n->getLong(), appendTo, pos);
-        break;
-    case Formattable::kInt64:
-        format(n->getInt64(), appendTo, pos);
-        break;
-    default:
-        status = U_INVALID_FORMAT_ERROR;
-        break;
+    return appendTo;
+}
+
+// -------------------------------------x
+// Formats the number object and save the format
+// result in the toAppendTo string buffer.
+
+UnicodeString&
+NumberFormat::format(const Formattable& obj,
+                        UnicodeString& appendTo,
+                        FieldPositionIterator* posIter,
+                        UErrorCode& status) const
+{
+    if (U_FAILURE(status)) return appendTo;
+
+    ArgExtractor arg(*this, obj, status);
+    const Formattable *n = arg.number();
+
+    if (n->isNumeric() && n->getDigitList() != NULL) {
+        // Decimal Number
+        format(*n->getDigitList(), appendTo, posIter, status);
+    } else {
+        switch (n->getType()) {
+        case Formattable::kDouble:
+            format(n->getDouble(), appendTo, posIter, status);
+            break;
+        case Formattable::kLong:
+            format(n->getLong(), appendTo, posIter, status);
+            break;
+        case Formattable::kInt64:
+            format(n->getInt64(), appendTo, posIter, status);
+            break;
+        default:
+            status = U_INVALID_FORMAT_ERROR;
+            break;
+        }
     }
 
-    if (setCurr) {
-        UErrorCode ok = U_ZERO_ERROR;
-        nonconst->setCurrency(save, ok); // always restore currency
-    }
     return appendTo;
 }
 
 // -------------------------------------
 
-UnicodeString& 
+UnicodeString&
 NumberFormat::format(int64_t number,
                      UnicodeString& appendTo,
                      FieldPosition& pos) const
@@ -413,7 +650,7 @@ NumberFormat::createCurrencyInstance(UErrorCode& status)
 
 NumberFormat* U_EXPORT2
 NumberFormat::createCurrencyInstance(const Locale& inLocale, UErrorCode& status)
-{  
+{
     return createInstance(inLocale, kCurrencyStyle, status);
 }
 
@@ -468,20 +705,6 @@ NumberFormat::getAvailableLocales(int32_t& count)
 //-------------------------------------------
 
 #if !UCONFIG_NO_SERVICE
-static ICULocaleService* gService = NULL;
-
-/**
- * Release all static memory held by numberformat.  
- */
-U_CDECL_BEGIN
-static UBool U_CALLCONV numfmt_cleanup(void) {
-    if (gService) {
-        delete gService;
-        gService = NULL;
-    }
-    return TRUE;
-}
-U_CDECL_END
 
 // -------------------------------------
 
@@ -501,7 +724,7 @@ private:
     Hashtable* _ids;
 
 public:
-    NFFactory(NumberFormatFactory* delegate) 
+    NFFactory(NumberFormatFactory* delegate)
         : LocaleKeyFactory(delegate->visible() ? VISIBLE : INVISIBLE)
         , _delegate(delegate)
         , _ids(NULL)
@@ -533,7 +756,7 @@ public:
 
 protected:
     /**
-     * Return the set of ids that this factory supports (visible or 
+     * Return the set of ids that this factory supports (visible or
      * otherwise).  This can be called often and might need to be
      * cached if it is expensive to create.
      */
@@ -584,7 +807,7 @@ public:
 
 // -------------------------------------
 
-static ICULocaleService* 
+static ICULocaleService*
 getNumberFormatService(void)
 {
     UBool needInit;
@@ -841,6 +1064,8 @@ NumberFormat::makeInstance(const Locale& desiredLocale,
             // fall-through
 
         case kCurrencyStyle:
+        case kIsoCurrencyStyle: // do not support plural formatting here
+        case kPluralCurrencyStyle:
             f = new Win32NumberFormat(desiredLocale, curr, status);
 
             if (U_SUCCESS(status)) {
@@ -849,7 +1074,7 @@ NumberFormat::makeInstance(const Locale& desiredLocale,
 
             delete f;
             break;
-            
+
         default:
             break;
         }
@@ -861,6 +1086,12 @@ NumberFormat::makeInstance(const Locale& desiredLocale,
     UnicodeString pattern;
     UResourceBundle *resource = ures_open((char *)0, desiredLocale.getName(), &status);
     UResourceBundle *numberPatterns = ures_getByKey(resource, DecimalFormat::fgNumberPatterns, NULL, &status);
+    NumberingSystem *ns = NULL;
+    UBool deleteSymbols = TRUE;
+    UHashtable * cache = NULL;
+    int32_t hashKey;
+    UBool getCache = FALSE;
+    UBool deleteNS = FALSE;
 
     if (U_FAILURE(status)) {
         // We don't appear to have resource data available -- use the last-resort data
@@ -874,7 +1105,7 @@ NumberFormat::makeInstance(const Locale& desiredLocale,
     else {
         // If not all the styled patterns exists for the NumberFormat in this locale,
         // sets the status code to failure and returns nil.
-        if (ures_getSize(numberPatterns) < (int32_t)(sizeof(gLastResortNumberPatterns)/sizeof(gLastResortNumberPatterns[0]))) {
+        if (ures_getSize(numberPatterns) < (int32_t)(sizeof(gLastResortNumberPatterns)/sizeof(gLastResortNumberPatterns[0])) -2 ) { //minus 2: ISO and plural
             status = U_INVALID_FORMAT_ERROR;
             goto cleanup;
         }
@@ -883,29 +1114,162 @@ NumberFormat::makeInstance(const Locale& desiredLocale,
         symbolsToAdopt = new DecimalFormatSymbols(desiredLocale, status);
 
         int32_t patLen = 0;
-        const UChar *patResStr = ures_getStringByIndex(numberPatterns, (int32_t)style, &patLen, &status);
+
+        /* for ISOCURRENCYSTYLE and PLURALCURRENCYSTYLE,
+         * the pattern is the same as the pattern of CURRENCYSTYLE
+         * but by replacing the single currency sign with
+         * double currency sign or triple currency sign.
+         */
+        int styleInNumberPattern = ((style == kIsoCurrencyStyle ||
+                                     style == kPluralCurrencyStyle) ?
+                                    kCurrencyStyle : style);
+
+        const UChar *patResStr = ures_getStringByIndex(numberPatterns, (int32_t)styleInNumberPattern, &patLen, &status);
+
         // Creates the specified decimal format style of the desired locale.
         pattern.setTo(TRUE, patResStr, patLen);
     }
     if (U_FAILURE(status) || symbolsToAdopt == NULL) {
         goto cleanup;
     }
-    if(style==kCurrencyStyle){
+    if(style==kCurrencyStyle || style == kIsoCurrencyStyle){
         const UChar* currPattern = symbolsToAdopt->getCurrencyPattern();
         if(currPattern!=NULL){
             pattern.setTo(currPattern, u_strlen(currPattern));
         }
     }
-    f = new DecimalFormat(pattern, symbolsToAdopt, status);
-    if (U_FAILURE(status) || f == NULL) {
+
+    // Use numbering system cache hashtable
+    UMTX_CHECK(&nscacheMutex, (UBool)(cache != NumberingSystem_cache), getCache);
+    if (getCache) {
+        umtx_lock(&nscacheMutex);
+        cache = NumberingSystem_cache;
+        umtx_unlock(&nscacheMutex);
+    }
+
+    // Check cache we got, create if non-existant
+    status = U_ZERO_ERROR;
+    if (cache == NULL) {
+        cache = uhash_open(uhash_hashLong,
+                           uhash_compareLong,
+                           NULL,
+                           &status);
+
+        if (cache == NULL || U_FAILURE(status)) {
+            // cache not created - out of memory
+            cache = NULL;
+        }
+        else {
+            // cache created
+            uhash_setValueDeleter(cache, deleteNumberingSystem);
+
+            // set final NumberingSystem_cache value
+            UHashtable* h = NULL;
+
+            UMTX_CHECK(&nscacheMutex, (UBool)(h != NumberingSystem_cache), getCache);
+            if (getCache) {
+                umtx_lock(&nscacheMutex);
+                h = NumberingSystem_cache;
+                umtx_unlock(&nscacheMutex);
+            }
+            if (h == NULL) {
+                umtx_lock(&nscacheMutex);
+                NumberingSystem_cache = h = cache;
+                cache = NULL;
+                ucln_i18n_registerCleanup(UCLN_I18N_NUMFMT, numfmt_cleanup);
+                umtx_unlock(&nscacheMutex);
+            }
+
+            if(cache != NULL) {
+              uhash_close(cache);
+            }
+            cache = h;
+        }
+    }
+
+    // Get cached numbering system
+    if (cache != NULL) {
+        hashKey = desiredLocale.hashCode();
+
+        umtx_lock(&nscacheMutex);
+        ns = (NumberingSystem *)uhash_iget(cache, hashKey);
+        if (ns == NULL) {
+            ns = NumberingSystem::createInstance(desiredLocale,status);
+            uhash_iput(cache, hashKey, (void*)ns, &status);
+        }
+        umtx_unlock(&nscacheMutex);
+    }
+    else {
+        ns = NumberingSystem::createInstance(desiredLocale,status);
+        deleteNS = TRUE;
+    }
+
+    // check results of getting a numbering system
+    if ((ns == NULL) || (U_FAILURE(status))) {
         goto cleanup;
     }
 
+    if (ns->isAlgorithmic()) {
+        UnicodeString nsDesc;
+        UnicodeString nsRuleSetGroup;
+        UnicodeString nsRuleSetName;
+        Locale nsLoc;
+        URBNFRuleSetTag desiredRulesType = URBNF_NUMBERING_SYSTEM;
+
+        nsDesc.setTo(ns->getDescription());
+        int32_t firstSlash = nsDesc.indexOf(gSlash);
+        int32_t lastSlash = nsDesc.lastIndexOf(gSlash);
+        if ( lastSlash > firstSlash ) {
+            char nsLocID[ULOC_FULLNAME_CAPACITY];
+
+            nsDesc.extract(0,firstSlash,nsLocID,ULOC_FULLNAME_CAPACITY,US_INV);
+            nsRuleSetGroup.setTo(nsDesc,firstSlash+1,lastSlash-firstSlash-1);
+            nsRuleSetName.setTo(nsDesc,lastSlash+1);
+
+            nsLoc = Locale::createFromName(nsLocID);
+
+            UnicodeString SpelloutRules = UNICODE_STRING_SIMPLE("SpelloutRules");
+            if ( nsRuleSetGroup.compare(SpelloutRules) == 0 ) {
+                desiredRulesType = URBNF_SPELLOUT;
+            }
+        } else {
+            nsLoc = desiredLocale;
+            nsRuleSetName.setTo(nsDesc);
+        }
+
+        RuleBasedNumberFormat *r = new RuleBasedNumberFormat(desiredRulesType,nsLoc,status);
+
+        if (U_FAILURE(status) || r == NULL) {
+            goto cleanup;
+        }
+        r->setDefaultRuleSet(nsRuleSetName,status);
+        f = (NumberFormat *) r;
+
+    } else {
+        // replace single currency sign in the pattern with double currency sign
+        // if the style is kIsoCurrencyStyle
+        if (style == kIsoCurrencyStyle) {
+            pattern.findAndReplace(gSingleCurrencySign, gDoubleCurrencySign);
+        }
+
+        f = new DecimalFormat(pattern, symbolsToAdopt, style, status);
+        if (U_FAILURE(status) || f == NULL) {
+            goto cleanup;
+        }
+        deleteSymbols = FALSE;
+    }
+
     f->setLocaleIDs(ures_getLocaleByType(numberPatterns, ULOC_VALID_LOCALE, &status),
                     ures_getLocaleByType(numberPatterns, ULOC_ACTUAL_LOCALE, &status));
+
 cleanup:
     ures_close(numberPatterns);
     ures_close(resource);
+
+    if (deleteNS && ns) {
+        delete ns;
+    }
+
     if (U_FAILURE(status)) {
         /* If f exists, then it will delete the symbols */
         if (f==NULL) {
@@ -920,6 +1284,9 @@ cleanup:
         status = U_MEMORY_ALLOCATION_ERROR;
         f = NULL;
     }
+    if (deleteSymbols && symbolsToAdopt != NULL) {
+        delete symbolsToAdopt;
+    }
     return f;
 }
 
